# %%
import ROOT
import numpy as np
# %jsroot on
import atlasplots as aplt
import pyfunctors
import const

ROOT.gInterpreter.ResetAll()
ROOT.gInterpreter.Declare('#include "cppfunctors.hxx"')
ROOT.EnableImplicitMT(24)
ROOT.gROOT.SetBatch()

aplt.set_atlas_style()

verbosity = ROOT.Experimental.RLogScopedVerbosity(ROOT.Detail.RDF.RDFLogChannel(), ROOT.Experimental.ELogLevel.kInfo)

roi = 'run2'
yoi = const.YEAR_DICT[roi]
# yoi = []
text_on_plot = const.TEXTS_ON_PLOT[roi]

coi = [
    # 'ggHtt',
    # 'VBFHtt',
    # 'WHtt',     
    # 'ZHtt',       
    'ttbar_allhad',
    'ttbar_lep',
    'singletop',     
    'diboson',           
    'W',         
    'Sh_Zee',      
    'Sh_Zmumu',
    # 'Dijet',   
    'Sh_Ztt',
    # 'Sh_Ztt_lowmass', 
    'data',   
]

idoi = {}
for ayear in yoi:
    idoi[ayear] = [adsid for achannel in coi for adsid in const.sample_dicts[ayear][achannel]]

# preprocess to get event weight
dfs_data = {}
dfs_data[18] = ROOT.RDF.Experimental.FromSpec('./specs/data_18_spec628.json').Define("final_weight", '1.0')
dfs_data[17] = ROOT.RDF.Experimental.FromSpec('./specs/data_17_spec628.json').Define("final_weight", '1.0')
dfs_data[15] = ROOT.RDF.Experimental.FromSpec('./specs/data_15_spec628.json').Define("final_weight", '1.0')
dfs_data[16] = ROOT.RDF.Experimental.FromSpec('./specs/data_16_spec628.json').Define("final_weight", '1.0')

dfs_mc = {}
dfs_mc[15]     = pyfunctors.FromSpecSingle('./specs/mc20a_15_spec628.json')
dfs_mc[16]     = pyfunctors.FromSpecSingle('./specs/mc20a_16_spec628.json')
dfs_mc[17]     = pyfunctors.FromSpecSingle('./specs/mc20d_17_spec628.json')
dfs_mc[18]     = pyfunctors.FromSpecSingle('./specs/mc20e_18_spec628.json')

dfs_of_interest = {}
for year in yoi:
    dfs_of_interest[year] = {}
    for dsid in dfs_mc[year].keys():
        if dsid in idoi[year]:
            dfs_of_interest[year][dsid] = ROOT.getFinalWeight(dfs_mc[year][dsid], const.DATA_LUMI[year])
    if 'data' in coi:
        dfs_of_interest[year]['data'] = dfs_data[year]

#event selections:
for ayear in dfs_of_interest.keys():
    trigger_c_str_vec = ROOT.std.vector(ROOT.std.string)(const.TRIGGER_DICT[ayear])
    for dsid in dfs_of_interest[ayear].keys():
        # dfs_of_interest[years][dsid] = ROOT.analysisMain(dfs_of_interest[years][dsid])
        dfs_of_interest[ayear][dsid] = ROOT.analysisMain(ROOT.filterTriggers(dfs_of_interest[ayear][dsid], trigger_c_str_vec))



hs1 = pyfunctors.fillHistos(
    dfs_of_interest, 
    'omega_noscale', 
    'final_weight', 
    pre_filter_expr='mutau_colin_p4.pt() > 100 && corr_mu_p4.pt() > 10 && leading_tau_dpst > 0.1',
    hist_model = ROOT.RDF.TH1DModel("", "", 40, -2, 2),
    post_filter_expr='mutau_colin_p4.M() > 40 && (!event_contain_b) && (leading_tau_q * corr_mu_q < 0)',
    yoi = yoi,
)

fig1, (ax11, ax21), otr1 = pyfunctors.makeRatioPlt(hs1, "omega_noscale", yoi=yoi, coi = coi, text = text_on_plot)
# ax1.set_yscale('log')
ax11.set_ylim(0, 500)
l11 = ROOT.TLine(-0.1, 0, -0.1, 500)
ax11.plot(l11)
l21 = ROOT.TLine(0.4, 0, 0.4, 500)
ax11.plot(l21)
fig1.canvas.Draw()
fig1.canvas.SaveAs('plot.pdf')